// Copyright 1999-2017. Parallels IP Holdings GmbH. All Rights Reserved.

(function (cssPrefix) {

    PleskExt.Docker.View = Class.create(Jsw.Component, {
        _initConfiguration: function ($super, config) {
            $super(config);

            this._cls = cssPrefix + 'docker-container-overview';
            this.urls = this._getConfigParam('urls', {});
            this.container = this._getConfigParam('container', {});
            this.logs = new PleskExt.Docker.Logs({
                dataUrl: this.urls.logs
            });
        },

        view: function () {
            return [
                new Jsw.SmallTools({
                    operations: [{
                        componentType: 'Jsw.SmallButton',
                        title: this.lmsg('startButton'),
                        addCls: cssPrefix + 'sb-start',
                        hidden: this.container.running,
                        handler: this.start.bind(this)
                    }, {
                        componentType: 'Jsw.SmallButton',
                        title: this.lmsg('stopButton'),
                        addCls: cssPrefix + 'sb-stop',
                        hidden: !this.container.running,
                        handler: this.stop.bind(this)
                    }, {
                        componentType: 'Jsw.SmallButton',
                        title: this.lmsg('restartButton'),
                        addCls: cssPrefix + 'sb-restart',
                        hidden: !this.container.running,
                        handler: this.restart.bind(this)
                    }, {
                        componentType: 'Jsw.SmallButton',
                        title: this.lmsg('recreateButton'),
                        addCls: cssPrefix + 'sb-recreate',
                        handler: this.recreate.bind(this)
                    }, {
                        componentType: 'Jsw.bar.Separator'
                    }, {
                        componentType: 'Jsw.SmallButton',
                        title: this.lmsg('commitButton'),
                        addCls: cssPrefix + 'sb-upload',
                        handler: this.commit.bind(this)
                    }, {
                        componentType: 'Jsw.SmallButton',
                        title: this.lmsg('exportButton'),
                        addCls: cssPrefix + 'sb-download',
                        href: this.urls.export
                    }, {
                        componentType: 'Jsw.bar.Separator'
                    }, {
                        componentType: 'Jsw.SmallButton',
                        title: this.lmsg('removeButton'),
                        addCls: cssPrefix + 'sb-remove',
                        handler: this.remove.bind(this)
                    }]
                }),
                Jsw.createElement("h3", this.lmsg('consoleLog')),
                this.logs
            ];
        },

        render: function($super) {
            $super();

            this.logs.stop();
            Jsw.render(this._componentElement, this.view(), 'inner');
            if (this.container.running) {
                this.logs.start();
            }
        },

        commit: function () {
            new Jsw.AjaxPopupForm({
                cls: 'popup-panel',
                title: this.lmsg('commitButton'),
                url: this.urls.commit,
                locale: {
                    waitMsg: this.lmsg('wait')
                }
            });
        },

        start: function () {
            new Ajax.Request(this.urls.start, {
                method: 'post',
                parameters: {
                    id: this.container.id
                },
                onSuccess: function (transport) {
                    this._responseHandler(transport, function () {
                        this.container.running = true;
                        this.render();
                    }.bind(this));
                }.bind(this)
            });
        },

        stop: function () {
            new Ajax.Request(this.urls.stop, {
                method: 'post',
                parameters: {
                    id: this.container.id
                },
                onSuccess: function (transport) {
                    this._responseHandler(transport, function () {
                        this.container.running = false;
                        this.render();
                    }.bind(this));
                }.bind(this)
            });
        },

        restart: function () {
            new Ajax.Request(this.urls.restart, {
                method: 'post',
                parameters: {
                    id: this.container.id
                },
                onSuccess: function (transport) {
                    this._responseHandler(transport, function () {
                        this.render();
                    }.bind(this));
                }.bind(this)
            });
        },

        recreate: function () {
            new Jsw.AjaxPopupForm({
                cls: 'popup-panel',
                title: this.lmsg('recreateTitle', { name: this.container.name.escapeHTML() }),
                url: this.urls.recreate,
                locale: {
                    waitMsg: this.lmsg('wait')
                }
            });
        },

        remove: function () {
            Jsw.messageBox.show({
                type: Jsw.messageBox.TYPE_YESNO,
                subtype: 'delete',
                text: this.lmsg('removeConfirmation'),
                onYesClick: function () {
                    new Ajax.Request(this.urls.remove, {
                        method: 'post',
                        parameters: {
                            id: this.container.id
                        },
                        onSuccess: function () {
                            Jsw.redirect(this.urls.list);
                        }.bind(this)
                    });
                }.bind(this),
                buttonTitles: {
                    yes: this.lmsg('yes'),
                    no: this.lmsg('no')
                }
            });
        },

        _responseHandler: function (transport, successCallback) {
            Jsw.clearStatusMessages();

            var response = transport.responseText.evalJSON();
            if (response && response.status && response.message) {
                Jsw.addStatusMessage(response.status, response.message);

                if (response.status !== 'info') {
                    return;
                }
            }

            successCallback();
        }
    });

})(PleskExt.Docker.PleskExtPrefix);
