// Copyright 1999-2017. Parallels IP Holdings GmbH. All Rights Reserved.

(function (ce, cssPrefix) {

    PleskExt.Docker.Images = Class.create(Jsw.Component, {
        _initConfiguration: function ($super, config) {
            $super(config);

            this._cls = cssPrefix +'docker-images-catalog';
            this.urls = this._getConfigParam('urls', {});
            this.icons = this._getConfigParam('icons', {});
            this.data = this._getConfigParam('data', []);
            this.hubUrl = this._getConfigParam('hubUrl');
        },

        render: function ($super) {
            $super();

            Jsw.render(this._componentElement, new Jsw.SmallTools({
                "locale": {
                    'buttonSearch': this.lmsg('buttonSearch'),
                    'buttonResetSearch': this.lmsg('buttonResetSearch')
                },
                "quickSearchWide": true,
                "searchCollapsed": true,
                "searchOveral": "name",
                "searchFilters": {
                    "name": {
                        "title": this.lmsg('filterImageName')
                    },
                    "isLocal": {
                        "title": this.lmsg('filterImageRepository'),
                        "options": [{
                            "type": "value",
                            "value": "",
                            "label": this.lmsg('filterOptionImageRepositorAny')
                        }, {
                            "type": "value",
                            "value": "true",
                            "label": this.lmsg('filterOptionImageRepositorLocal')
                        }, {
                            "type": "value",
                            "value": "false",
                            "label": this.lmsg('filterOptionImageRepositorRemote')
                        }]
                    }
                },
                "searchHandler": function (form) {
                    this.search(form.serialize(true));
                }.bind(this),
                "resetSearchHandler": this.search.bind(this, {}),
                "operations": [
                    {
                        componentType: 'PleskExt.Docker.UploadButton',
                        id: 'buttonUploadImage',
                        title: this.lmsg('buttonUploadImage'),
                        description: '',
                        addCls: 'sb-upload',
                        uploadPopupParams: {
                            supportedExtensions: ['.tar', '.tar.gz', '.tgz', '.bzip', '.tar.xz', '.txz'],
                            id: 'fm-upload',
                            icon: this.icons.upload,
                            renderTo: 'fm-modal-dialog-box-upload',
                            cls: 'popup-panel',
                            url: this.urls['upload-image'],
                            locale: this._getConfigParam('locale').uploadImage,
                            completeCallback: function (e) {
                                Jsw.redirect(this.urls['list']);
                            }.bind(this)
                        }
                    }
                ]
            }));

            // add main container for list
            this._catalogElement = new Element('div', {'class': cssPrefix +'catalog'});
            this._componentElement.insert(this._catalogElement);

            this._initDisablerOverlay();

            this._redrawImages();
        },

        emptyView: function () {
            return '<p>' + this.lmsg('noAvailableCatalogItems') + '</p>';
        },

        listView: function (data) {
            return ce(".grid.grid-stretched.grid-divided-horizontal.grid-divided-vertical-solid.grid-xl-2.grid-xxl-3",
                data.map(this.itemView.bind(this))
            );
        },

        itemView: function (item) {
            var name = item.name + (item.tag ? ': ' + item.tag : '');
            if (!item.isLocal && this.hubUrl) {
                name = ce('div', name,
                    ce('a.' + cssPrefix + 'catalog-item-external[target="_blank"]', {
                        href: this.hubUrl + '/' + (item.isOfficial ? '_' : 'r') + '/' + name + '/'
                    })
                );
            }

            var cache = "";
            if (item.count && item.cache) {
                cache = ce(
                    "a.toggler#toggle-image-" + item.id,
                    this.lmsg('imagesCached', {count: item.count})
                );
            }

            return ce(".grid-col",
                ce("article."+ cssPrefix +"catalog-item",
                    ce("."+ cssPrefix +"catalog-item-start",
                        ce("figure."+ cssPrefix +"image",
                            ce("img", {"src": this.icons.default, "alt": ""})
                        ),
                        item.isOfficial ? ce('.label.label-success', this.lmsg('officialLabel')) : '',
                        item.starCount ? ce('.label.label-info', this.lmsg('starCount', {count: item.starCount})) : ''
                    ),
                    ce("."+ cssPrefix +"catalog-item-body",
                        ce("header."+ cssPrefix +"catalog-item-heading",
                            name
                        ),
                        ce("."+ cssPrefix +"catalog-item-content", item.description),
                        ce("footer."+ cssPrefix +"catalog-item-footer",
                            cache,
                            new Jsw.SplitButton({
                                "title": item.latestCached ? this.lmsg('installCached') : this.lmsg('install'),
                                "description": this.lmsg('installTooltip'),
                                "onclick": this.install.bind(this, item.name, item.tag),
                                "isMenuFlip": true,
                                "isScrollable": true,
                                "locale": {
                                    "loading": this.lmsg("loading")
                                },
                                "items": this.imageTags.bind(this, item)
                            })
                        )
                    )
                )
            );
        },

        _getCacheList: function (item) {
            return new Jsw.List({
                id: 'docker-image-cache-list-' + item.id,
                pageable: false,
                data: {
                    data: item.cache
                },
                operations: [
                    {
                        componentType: 'Jsw.SmallButton',
                        id: 'buttonCleanCache-' + item.id,
                        title: this.lmsg('cleanCache'),
                        description: '',
                        addCls: 'sb-delete',
                        handler: this.remove.bind(this, item, null, true)
                    }
                ],
                columns: [
                    {
                        sortable: false,
                        dataIndex: 'tag',
                        renderer: function (listItem) {
                            return listItem.tag ? listItem.tag : 'latest';
                        }
                    },
                    {
                        sortable: false,
                        dataIndex: 'size',
                        renderer: function (listItem) {
                            return listItem.size;
                        }
                    },
                    {
                        sortable: false,
                        dataIndex: 'size',
                        renderer: function (listItem) {
                            var tag = listItem.tag.replace(/\.|-|_/g, '');
                            return "<span id='image-cache-item-" + item.id + tag + "' style='cursor:pointer;'>" +
                                "<img alt='" + this.lmsg('remove') + "' src='" + this.icons.remove + "' />" +
                                "</span>";
                        }.bind(this)
                    }
                ],
                onRedraw: function () {
                    Object.keys(item.cache).forEach(function (index) {
                        var cacheItem = item.cache[index],
                            tag = cacheItem.tag.replace(/\.|-|_/g, '');
                        $(Jsw.DynamicPopupHint.windowId)
                            .down('#image-cache-item-' + item.id + tag)
                            .observe('click', this.remove.bind(this, item, cacheItem.tag, false));
                    }.bind(this))
                }.bind(this)
            });
        },

        _redrawImages: function () {
            Jsw.render(this._catalogElement, this.data.length ? this.listView(this.data) : this.emptyView(), 'inner');
            Jsw.Tooltip.initData(this._catalogElement);
            this.data.forEach(function (item) {
                if ($('toggle-image-' + item.id)) {
                    new Jsw.DynamicPopupHint.Instance({
                        title: this.lmsg('cachedImagesOf', {image: item.name}),
                        target: this._componentElement.down('#toggle-image-' + item.id),
                        placement: 'left',
                        content: this._getCacheList(item)
                    });
                }
            }.bind(this));
        },

        search: function (params) {
            this._disablerOverlay.show();
            this._disablerOverlay.clonePosition(this._catalogElement);
            new Ajax.Request(this.urls['images-list-data'], {
                method: 'post',
                onSuccess: this._onSearchSuccess.bind(this),
                onComplete: function () {
                    this._disablerOverlay.hide();
                }.bind(this),
                parameters: params
            });
        },

        _onSearchSuccess: function (transport) {
            this.data = transport.responseText.evalJSON();
            this._redrawImages();
        },

        install: function (name, tag, event) {
            var button = Jsw.getComponent(event.element().up('.btn-group').identify());

            new Ajax.Request(this.urls.pull, {
                method: 'post',
                parameters: {
                    name: name,
                    tag: tag
                },
                onSuccess: function (transport) {
                    var data = transport.responseText.evalJSON();
                    if (data.componentType) {
                        Jsw.getComponent('asyncProgressBarWrapper').progressDialog(data);
                    } else if (data.redirect) {
                        Jsw.redirect(data.redirect);
                    }
                },
                onCreate: function () {
                    button.disable();
                },
                onComplete: function () {
                    button.enable();
                }
            });
        },

        remove: function (item, tag, all) {
            Jsw.DynamicPopupHint.unset();
            var name = item.name,
                needAttention = false;
            if (name.indexOf(':') === -1 && !all) {
                name = [name, tag || 'latest'].join(':');
            }

            this._disablerOverlay.show();
            this._disablerOverlay.clonePosition(this._catalogElement);

            new Ajax.Request(this.urls['check-image-remove'], {
                method: 'post',
                parameters: {
                    name: item.name,
                    tag: tag,
                    all: all
                },
                onSuccess: function (transport) {
                    var data = transport.responseText.evalJSON();
                    needAttention = data.needAttention;
                }.bind(this),
                onComplete: function () {
                    this._disablerOverlay.hide();
                    this._onRemove(needAttention, name, item, all);
                }.bind(this)
            });
        },

        _onRemove: function (needAttention, name, item, all) {
            Jsw.messageBox.show({
                type: Jsw.messageBox.TYPE_YESNO,
                subtype: 'delete',
                text: this.lmsg(all ? 'cleanCacheConfirmation' : 'removeImageConfirmation', {image: name}),
                needAttention: needAttention,
                needAttentionText: this.lmsg('removeImageAttention'),
                needAttentionBlockSubmit: true,
                onYesClick: function () {
                    var index = this.data.indexOf(item);
                    this.data.splice(index, 1);
                    this._redrawImages();

                    this._disablerOverlay.show();
                    this._disablerOverlay.clonePosition(this._catalogElement);

                    Jsw.redirectPost(this.urls['image-remove'], {name: name, all: all});
                }.bind(this),
                buttonTitles: {
                    yes: Jsw.Locale.getSection('components.buttons').lmsg('yes'),
                    no: Jsw.Locale.getSection('components.buttons').lmsg('no')
                }
            });
        },

        /**
         * @param {Object} item
         * @returns {Promise}
         */
        imageTags: function (item) {
            return new Promise(function (resolve) {
                new Ajax.Request(this.urls['image-tags'], {
                    method: 'get',
                    parameters: {
                        name: item.name,
                        isOfficial: item.isOfficial
                    },
                    onSuccess: function (transport) {
                        var data = transport.responseText.evalJSON();
                        resolve(data.map(function (tag) {
                            return {
                                "title": tag.cached ? this.lmsg('tagCached', {tag: tag.name}) : tag.name,
                                "onclick": this.install.bind(this, item.name, tag.name)
                            };
                        }.bind(this)));
                    }.bind(this)
                });
            }.bind(this));
        }
    });

})(Jsw.createElement, PleskExt.Docker.PleskExtPrefix);
