// Copyright 1999-2017. Parallels IP Holdings GmbH. All Rights Reserved.

(function (ce) {
    PleskExt.Docker.UploadPopup = Class.create(Jsw.PopupForm, {
        _supportedExtensions: null,
        _completeCallback: null,
        _file: null,
        _url: null,
        _icon: null,
        _tasksQueue: [],
        _currentRequest: null,

        _initConfiguration: function ($super, config) {
            $super(config);

            this._file = this._getConfigParam('file');
            this._supportedExtensions = this._getConfigParam('supportedExtensions');
            this._url = this._getConfigParam('url', '');
            this._icon = this._getConfigParam('icon');
            this._completeCallback = this._getConfigParam('completeCallback', function () {});
        },

        render: function ($super) {
            $super();
            this.setBoxType('form-box');
            this.setTitle(this.lmsg('title'));
        },

        _redrawProgress: function (taskId, loaded, total) {
            var percent = Math.ceil(loaded / total * 100),
                mbLiterals = Jsw.Locale.getSection('literals').lmsg('mega')
                    + Jsw.Locale.getSection('literals').lmsg('byte');
            Jsw.render($('task_' + taskId), ce('div', {style: 'display: flex'},
                this._icon ? ce('.progress-step-icon', ce('i.icon-md', ce('img', {src: this._icon}))) : '',
                ce('.progress-box.resource-usage', {style: 'flex: 1 1 auto;'},
                    ce('.progress-label', this.lmsg('description', {filename: this._file.name})),
                    ce('.progress-label',
                        this.lmsg('sizeProgress', {
                            loaded: PleskExt.Docker.Utils.bytesConvert(
                                loaded, null, PleskExt.Docker.SizeUnits.MEGABYTE
                            ) + mbLiterals,
                            total: PleskExt.Docker.Utils.bytesConvert(
                                total, null, PleskExt.Docker.SizeUnits.MEGABYTE
                            ) + mbLiterals
                        })
                    ),
                    ce('.progress', ce('.progress-bar', {style: 'width: ' + percent + '%;'})),
                    ce('.progress-label', this.lmsg('progress', {value: percent}))
                )), 'inner');
        },

        _checkExtension: function () {
            if (!this._supportedExtensions) {
                return true;
            }

            var matched = false;

            this._supportedExtensions.forEach(function (el) {
                if (this._file.name.match(new RegExp(el.replace(/\./g, '\\.') + '$', 'g'))) {
                    matched = true;
                }
            }.bind(this));

            return matched;
        },

        _showError: function (taskId, errorMessage) {
            Jsw.render($('task_' + taskId), ce('.progress-step.progress-step-danger',
                this._icon ? ce('.progress-step-icon', ce('i.icon-md', ce('img', {src: this._icon}))) : '',
                ce('.progress-step-body',
                    ce('.progress-step-status', ce(
                        'i.icon-warning',
                        {style: 'padding-right: 2px'}),
                        Jsw.Locale.getSection('statuses').lmsg('failed')
                    ),
                    ce('.progress-step-name', this.lmsg('description', {filename: this._file.name})),
                    ce('.progress-step-content', errorMessage)
                )), 'inner');
        },

        _showSuccess: function (taskId) {
            Jsw.render($('task_' + taskId), ce('.progress-step.progress-step-success',
                this._icon ? ce('.progress-step-icon', ce('i.icon-md', ce('img', {src: this._icon}))) : '',
                ce('.progress-step-body',
                    ce('.progress-step-status', ce('i.icon-ok', {style: 'padding-right: 2px'}), this.lmsg('done')),
                    ce('.progress-step-name', this.lmsg('description', {filename: this._file.name}))
                )), 'inner');
        },
        
        _createTask: function () {
            var taskId = Date.now();
            this._tasksQueue.push(taskId);
            if (!$('task_' + taskId)) {
                Jsw.render($(this._contentAreaId), ce('div#task_' + taskId));
            }
            
            return taskId;
        },

        _handleStatus: function (status, params) {
            var button = '';
            switch (status) {
                case 'success':
                    this._showSuccess.apply(this, params);
                    button = 'close';
                    break;
                case 'error':
                    this._showError.apply(this, params);
                    button = 'close';
                    break;
                default:
                    return;
            }

            var i = this._tasksQueue.indexOf(params[0]);
            if (i > -1) {
                this._tasksQueue.splice(i, 1);
            }

            if (!this._tasksQueue.length) {
                this.removeRightButtons();
                this.addRightButton(Jsw.Locale.getSection('components.buttons').lmsg(button), function () {
                    this.hide();
                    if (document.activeElement != document.body) {
                        document.activeElement.blur();
                    }
                }.bind(this), false, false);
            }
        },

        _finishTaskHandler: function (taskId, transport, exception) {
            if (exception) {
                this._handleStatus('error', [taskId, this.lmsg('errorOccurred')]);
                return;
            }

            var response = {};
            try {
                response = transport.responseText.evalJSON();
            } catch (err) {
            }

            if (response && response.status && response.status !== 'success' && response.messages) {
                var messages = [];
                $H(response.messages).each(function (pair) {
                    if (!response.messages.hasOwnProperty(pair.key)) {
                        return;
                    }
                    messages.push(pair.value);
                }.bind(this));
                if (!messages.length) {
                    messages.push(this.lmsg('errorOccurred'));
                }
                this._handleStatus('error', [taskId, messages.join('<br />')]);
                return;
            }

            this._handleStatus('success', [taskId]);

            if (document.activeElement != document.body) {
                document.activeElement.blur();
            }
            this._completeCallback(transport);
        },

        start: function () {
            var taskId = this._createTask();
            if (!this._checkExtension()) {
                this._handleStatus('error', [taskId, this.lmsg('unsupportedExtensions', {
                    extensions: this._supportedExtensions.join(', ').replace(/,\s&/g, '')
                })]);
                return;
            }
            
            this._currentRequest = new PleskExt.Docker.UploadAdapterAjax({
                handlerUrl: this._url,
                startUploadHandler: null,
                finishUploadHandler: this._finishTaskHandler.bind(this, taskId),
                progressUploadHandler: function (e) {
                    this._redrawProgress(taskId, e.loaded, e.total);
                }.bind(this)
            });
            this._currentRequest.upload(this._file);

            this.addRightButton(Jsw.Locale.getSection('components.buttons').lmsg('cancel'), function () {
                this._currentRequest.cancel();
                this.hide();
                if (document.activeElement != document.body) {
                    document.activeElement.blur();
                }
            }.bind(this), false, false);
        }
    });
})(Jsw.createElement);
